@@ -15,14 +15,16 @@ export PATH
 CMAKE_PREFIX_PATH="${QT_BASEPATH}/lib/cmake"
 export CMAKE_PREFIX_PATH
 
+export CMAKE_BUILD_PARALLEL_LEVEL="$(sysctl -n hw.ncpu)"
+
+export CMAKE_POLICY_VERSION_MINIMUM="3.5"
+
 cmake -GNinja -S. -Bbuild \
       -DCMAKE_BUILD_TYPE=RelWithDebInfo \
       -DCMAKE_INSTALL_PREFIX="nheko.temp" \
       -DHUNTER_ROOT="../.hunter" \
       -DHUNTER_ENABLED=ON -DBUILD_SHARED_LIBS=OFF \
       -DCMAKE_BUILD_TYPE=RelWithDebInfo -DHUNTER_CONFIGURATION_TYPES=RelWithDebInfo \
-      -DUSE_BUNDLED_OPENSSL=ON \
-      -DUSE_BUNDLED_KDSINGLEAPPLICATION=ON -DKDSingleApplication_STATIC=ON \
       -DQt6_DIR=${QT_BASEPATH}/lib/cmake \
       -DCI_BUILD=ON
 cmake --build build
@@ -31,7 +33,7 @@ cmake --install build
   git clone https://github.com/Nheko-Reborn/qt-jdenticon.git
   ( cd qt-jdenticon
     qmake
-    make -j 4
+    make -j "$CMAKE_BUILD_PARALLEL_LEVEL"
     cp libqtjdenticon.dylib ../../nheko.temp/nheko.app/Contents/MacOS
   )
   # "$(brew --prefix qt6)/bin/macdeployqt" nheko.app -always-overwrite -qmldir=../resources/qml/
