@@ -807,7 +807,35 @@ ChatPage::handleSyncResponse(const mtx::responses::Sync &res, const std::string
         cache::client()->saveState(res);
         olm::handle_to_device_messages(res.to_device.events);
 
-        auto updates = cache::getRoomInfo(cache::client()->roomsWithStateUpdates(res));
+        // reject forbidden invites
+        if (!res.rooms.invite.empty()) {
+            if (auto ev =
+                  cache::client()->getAccountData(mtx::events::EventType::NhekoInvitePermissions)) {
+                const auto &invitePerms = std::get<mtx::events::AccountDataEvent<
+                  mtx::events::account_data::nheko_extensions::InvitePermissions>>(*ev)
+                                            .content;
+
+                for (const auto &[roomid, invite] : res.rooms.invite) {
+                    std::string_view inviter = "";
+                    for (const auto &memberEv : invite.invite_state) {
+                        if (auto member =
+                              std::get_if<mtx::events::StrippedEvent<mtx::events::state::Member>>(
+                                &memberEv)) {
+                            if (member->content.membership ==
+                                  mtx::events::state::Membership::Invite &&
+                                member->state_key == http::client()->user_id().to_string()) {
+                                inviter = member->sender;
+                                break;
+                            }
+                        }
+                    }
+
+                    if (!invitePerms.invite_allowed(roomid, inviter)) {
+                        leaveRoom(QString::fromStdString(roomid), "");
+                    }
+                }
+            }
+        }
 
         emit syncUI(std::move(res));
 
@@ -1084,14 +1112,14 @@ void
 ChatPage::kickUser(const QString &room, QString userid, QString reason)
 {
     bool confirmed;
-    reason =
-      QInputDialog::getText(nullptr,
-                            tr("Reason for the kick"),
-                            tr("Enter reason for kicking %1 (%2) or hit enter for no reason:")
-                              .arg(cache::displayName(room, userid), userid),
-                            QLineEdit::Normal,
-                            reason,
-                            &confirmed);
+    reason = QInputDialog::getText(
+      nullptr,
+      tr("Reason for the kick"),
+      tr("Enter reason for kicking %1 (%2) or hit enter for no reason:")
+        .arg(cache::displayName(room, userid).toHtmlEscaped(), userid.toHtmlEscaped()),
+      QLineEdit::Normal,
+      reason,
+      &confirmed);
     if (!confirmed) {
         return;
     }
@@ -1113,14 +1141,14 @@ void
 ChatPage::banUser(const QString &room, QString userid, QString reason)
 {
     bool confirmed;
-    reason =
-      QInputDialog::getText(nullptr,
-                            tr("Reason for the ban"),
-                            tr("Enter reason for banning %1 (%2) or hit enter for no reason:")
-                              .arg(cache::displayName(room, userid), userid),
-                            QLineEdit::Normal,
-                            reason,
-                            &confirmed);
+    reason = QInputDialog::getText(
+      nullptr,
+      tr("Reason for the ban"),
+      tr("Enter reason for banning %1 (%2) or hit enter for no reason:")
+        .arg(cache::displayName(room, userid).toHtmlEscaped(), userid.toHtmlEscaped()),
+      QLineEdit::Normal,
+      reason,
+      &confirmed);
     if (!confirmed) {
         return;
     }
@@ -1144,7 +1172,8 @@ ChatPage::unbanUser(const QString &room, QString userid, QString reason)
     if (QMessageBox::question(nullptr,
                               tr("Confirm unban"),
                               tr("Do you really want to unban %1 (%2)?")
-                                .arg(cache::displayName(room, userid), userid)) != QMessageBox::Yes)
+                                .arg(cache::displayName(room, userid).toHtmlEscaped(),
+                                     userid.toHtmlEscaped())) != QMessageBox::Yes)
         return;
 
     http::client()->unban_user(
@@ -1584,112 +1613,19 @@ ChatPage::startChat(QString userid, std::optional<bool> encryptionEnabled)
     emit ChatPage::instance()->createRoom(req);
 }
 
-static QString
-mxidFromSegments(QStringView sigil, QStringView mxid)
-{
-    if (mxid.isEmpty())
-        return QString();
-
-    auto mxid_ = QUrl::fromPercentEncoding(mxid.toUtf8());
-
-    if (sigil == u"u") {
-        return "@" + mxid_;
-    } else if (sigil == u"roomid") {
-        return "!" + mxid_;
-    } else if (sigil == u"r") {
-        return "#" + mxid_;
-        //} else if (sigil == "group") {
-        //        return "+" + mxid_;
-    } else {
-        return QString();
-    }
-}
-
 bool
 ChatPage::handleMatrixUri(QString uri)
 {
     nhlog::ui()->info("Received uri! {}", uri.toStdString());
-    QUrl uri_{uri};
 
-    // Convert matrix.to URIs to proper format
-    if (uri_.scheme() == QLatin1String("https") && uri_.host() == QLatin1String("matrix.to")) {
-        QString p = uri_.fragment(QUrl::FullyEncoded);
-        if (p.startsWith(QLatin1String("/")))
-            p.remove(0, 1);
+    auto m = utils::parseMatrixUri(uri);
 
-        auto temp = p.split(QStringLiteral("?"));
-        QString query;
-        if (temp.size() >= 2)
-            query = QUrl::fromPercentEncoding(temp.takeAt(1).toUtf8());
-
-        temp            = temp.first().split(QStringLiteral("/"));
-        auto identifier = QUrl::fromPercentEncoding(temp.takeFirst().toUtf8());
-        QString eventId = QUrl::fromPercentEncoding(temp.join('/').toUtf8());
-        if (!identifier.isEmpty()) {
-            if (identifier.startsWith(QLatin1String("@"))) {
-                QByteArray newUri = "matrix:u/" + QUrl::toPercentEncoding(identifier.remove(0, 1));
-                if (!query.isEmpty())
-                    newUri.append("?" + query.toUtf8());
-                return handleMatrixUri(QUrl::fromEncoded(newUri));
-            } else if (identifier.startsWith(QLatin1String("#"))) {
-                QByteArray newUri = "matrix:r/" + QUrl::toPercentEncoding(identifier.remove(0, 1));
-                if (!eventId.isEmpty())
-                    newUri.append("/e/" + QUrl::toPercentEncoding(eventId.remove(0, 1)));
-                if (!query.isEmpty())
-                    newUri.append("?" + query.toUtf8());
-                return handleMatrixUri(QUrl::fromEncoded(newUri));
-            } else if (identifier.startsWith(QLatin1String("!"))) {
-                QByteArray newUri =
-                  "matrix:roomid/" + QUrl::toPercentEncoding(identifier.remove(0, 1));
-                if (!eventId.isEmpty())
-                    newUri.append("/e/" + QUrl::toPercentEncoding(eventId.remove(0, 1)));
-                if (!query.isEmpty())
-                    newUri.append("?" + query.toUtf8());
-                return handleMatrixUri(QUrl::fromEncoded(newUri));
-            }
-        }
+    if (!m) {
+        nhlog::ui()->info("failed to parse uri! {}", uri.toStdString());
+        return false;
     }
 
-    // non-matrix URIs are not handled by us, return false
-    if (uri_.scheme() != QLatin1String("matrix"))
-        return false;
-
-    auto tempPath = uri_.path(QUrl::ComponentFormattingOption::FullyEncoded);
-    if (tempPath.startsWith('/'))
-        tempPath.remove(0, 1);
-    auto segments = QStringView(tempPath).split('/');
-
-    if (segments.size() != 2 && segments.size() != 4)
-        return false;
-
-    auto sigil1 = segments[0];
-    auto mxid1  = mxidFromSegments(sigil1, segments[1]);
-    if (mxid1.isEmpty())
-        return false;
-
-    QString mxid2;
-    if (segments.size() == 4 && segments[2] == QStringView(u"e")) {
-        if (segments[3].isEmpty())
-            return false;
-        else
-            mxid2 = "$" + QUrl::fromPercentEncoding(segments[3].toUtf8());
-    }
-
-    std::vector<std::string> vias;
-    QString action;
-
-    auto items =
-      uri_.query(QUrl::ComponentFormattingOption::FullyEncoded).split('&', Qt::SkipEmptyParts);
-    for (QString item : std::as_const(items)) {
-        nhlog::ui()->info("item: {}", item.toStdString());
-
-        if (item.startsWith(QLatin1String("action="))) {
-            action = item.remove(QStringLiteral("action="));
-        } else if (item.startsWith(QLatin1String("via="))) {
-            vias.push_back(QUrl::fromPercentEncoding(item.remove(QStringLiteral("via=")).toUtf8())
-                             .toStdString());
-        }
-    }
+    const auto &[sigil1, mxid1, sigil2, mxid2, action, vias] = *m;
 
     if (sigil1 == u"u") {
         if (action.isEmpty()) {
