@@ -1,4 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -8,7 +8,7 @@
 
 #include <mtx/events/mscs/image_packs.hpp>
 
-class CombinedImagePackModel : public QAbstractListModel
+class CombinedImagePackModel final : public QAbstractListModel
 {
     Q_OBJECT
 public:
@@ -18,21 +18,14 @@ public:
         ShortCode,
         Body,
         PackName,
-        OriginalRow,
+        Unicode,
     };
 
-    CombinedImagePackModel(const std::string &roomId, bool stickers, QObject *parent = nullptr);
+    CombinedImagePackModel(const std::string &roomId, QObject *parent = nullptr);
     QHash<int, QByteArray> roleNames() const override;
     int rowCount(const QModelIndex &parent = QModelIndex()) const override;
     QVariant data(const QModelIndex &index, int role) const override;
 
-    mtx::events::msc2545::PackImage imageAt(int row)
-    {
-        if (row < 0 || static_cast<size_t>(row) >= images.size())
-            return {};
-        return images.at(static_cast<size_t>(row)).image;
-    }
-
 private:
     std::string room_id;
 
