@@ -105,6 +105,10 @@ CommandCompleter::data(const QModelIndex &index, int role) const
                 return QStringLiteral("/ignore");
             case Unignore:
                 return QStringLiteral("/unignore");
+            case BlockInvites:
+                return QStringLiteral("/blockinvites");
+            case AllowInvites:
+                return QStringLiteral("/allowinvites");
             default:
                 return {};
             }
@@ -186,6 +190,10 @@ CommandCompleter::data(const QModelIndex &index, int role) const
                 return QStringLiteral("/ignore <@userid>");
             case Unignore:
                 return QStringLiteral("/unignore <@userid>");
+            case BlockInvites:
+                return QStringLiteral("/blockinvites <@userid>|<!roomid>|<servername>|all");
+            case AllowInvites:
+                return QStringLiteral("/allowinvites <@userid>|<!roomid>|<servername>|all");
             default:
                 return {};
             }
@@ -207,13 +215,17 @@ CommandCompleter::data(const QModelIndex &index, int role) const
             case Invite:
                 return tr("Invite a user into the current room. Reason is optional.");
             case Kick:
-                return tr("Kick a user from the current room. Reason is optional.");
+                return tr("Kick a user from the current room. Reason is optional. If user is left "
+                          "out, will try to kick the sender you are replying to.");
             case Ban:
-                return tr("Ban a user from the current room. Reason is optional.");
+                return tr("Ban a user from the current room. Reason is optional. If user is left "
+                          "out, will try to ban the sender you are replying to.");
             case Unban:
-                return tr("Unban a user in the current room. Reason is optional.");
+                return tr("Unban a user in the current room. Reason is optional. If user is left "
+                          "out, will try to unban the sender you are replying to.");
             case Redact:
-                return tr("Redact an event or all locally cached messages of a user.");
+                return tr("Redact an event by event id or that you are replying to or all locally "
+                          "cached messages of a user.");
             case Roomnick:
                 return tr("Change your displayname in this room.");
             case Shrug:
@@ -267,6 +279,12 @@ CommandCompleter::data(const QModelIndex &index, int role) const
                 return tr("Ignore a user.");
             case Unignore:
                 return tr("Stop ignoring a user.");
+            case BlockInvites:
+                return tr("Block all invites from a user, a server, to a specific room or set the "
+                          "default.");
+            case AllowInvites:
+                return tr("Allow all invites from a user, a server, to a specific room or set the "
+                          "default.");
             default:
                 return {};
             }
