@@ -32,16 +32,17 @@ MxcImageProvider::MxcImageProvider()
     timer->setInterval(std::chrono::hours(1));
     connect(timer, &QTimer::timeout, this, [] {
         QThreadPool::globalInstance()->start([] {
+            nhlog::net()->debug("Running media purge");
             QDir dir(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) +
                        "/media_cache",
                      "",
                      QDir::SortFlags(QDir::Name | QDir::IgnoreCase),
-                     QDir::Filter::Writable | QDir::Filter::NoDotAndDotDot | QDir::Filter::Files);
+                     QDir::Filter::Writable | QDir::Filter::NoDotAndDotDot | QDir::Filter::Files |
+                       QDir::Filter::Dirs);
 
-            auto files = dir.entryInfoList();
-            for (const auto &fileInfo : std::as_const(files)) {
+            auto handleFile = [](const QFileInfo &fileInfo) {
                 if (fileInfo.fileTime(QFile::FileTime::FileAccessTime)
-                      .daysTo(QDateTime::currentDateTime()) > 30) {
+                      .daysTo(QDateTime::currentDateTime()) > 14) {
                     if (QFile::remove(fileInfo.absoluteFilePath()))
                         nhlog::net()->debug("Deleted stale media '{}'",
                                             fileInfo.absoluteFilePath().toStdString());
@@ -49,6 +50,24 @@ MxcImageProvider::MxcImageProvider()
                         nhlog::net()->warn("Failed to delete stale media '{}'",
                                            fileInfo.absoluteFilePath().toStdString());
                 }
+            };
+
+            auto files = dir.entryInfoList();
+            for (const auto &fileInfo : std::as_const(files)) {
+                if (fileInfo.isDir()) {
+                    // handle one level of legacy directories
+                    auto nestedDir = QDir(fileInfo.absoluteFilePath(),
+                                          "",
+                                          QDir::SortFlags(QDir::Name | QDir::IgnoreCase),
+                                          QDir::Filter::Writable | QDir::Filter::NoDotAndDotDot |
+                                            QDir::Filter::Files)
+                                       .entryInfoList();
+                    for (const auto &nestedFile : std::as_const(nestedDir)) {
+                        handleFile(nestedFile);
+                    }
+                } else {
+                    handleFile(fileInfo);
+                }
             }
         });
     });
@@ -72,7 +91,7 @@ MxcImageProvider::requestImageResponse(const QString &id, const QSize &requested
         auto query     = QStringView(id).mid(queryStart + 1);
         auto queryBits = query.split('&');
 
-        for (auto b : queryBits) {
+        for (auto b : std::as_const(queryBits)) {
             if (b == QStringView(u"scale")) {
                 crop = false;
             } else if (b.startsWith(QStringView(u"radius="))) {
@@ -260,9 +279,10 @@ MxcImageProvider::download(const QString &id,
                   }
               }
               image.setText(QStringLiteral("mxc url"), "mxc://" + id);
-              if (image.save(fileInfo.absoluteFilePath(), "png"))
+              if (image.save(fileInfo.absoluteFilePath(), "png")) {
+                  utils::markFileAsFromWeb(fileInfo.absoluteFilePath());
                   nhlog::ui()->debug("Wrote: {}", fileInfo.absoluteFilePath().toStdString());
-              else
+              } else
                   nhlog::ui()->debug("Failed to write: {}",
                                      fileInfo.absoluteFilePath().toStdString());
 
@@ -337,6 +357,7 @@ MxcImageProvider::download(const QString &id,
                   }
                   f.write(tempData.data(), tempData.size());
                   f.close();
+                  utils::markFileAsFromWeb(fileInfo.absoluteFilePath());
 
                   if (encryptionInfo) {
                       tempData = mtx::crypto::to_string(
