@@ -11,10 +11,11 @@
 #include <QWindow>
 
 #include "TrayIcon.h"
+#include "UserSettingsPage.h"
 
-MsgCountComposedIcon::MsgCountComposedIcon(const QString &filename)
+MsgCountComposedIcon::MsgCountComposedIcon(const QIcon &icon)
   : QIconEngine()
-  , icon_{QIcon{filename}}
+  , icon_{icon}
 {
 }
 
@@ -41,8 +42,8 @@ MsgCountComposedIcon::paint(QPainter *painter,
     brush.setColor(backgroundColor);
 
     QFont f;
-    f.setPointSizeF(8);
-    f.setWeight(QFont::Thin);
+    f.setPixelSize(int(BubbleDiameter * 0.6));
+    f.setWeight(QFont::Bold);
 
     painter->setBrush(brush);
     painter->setPen(Qt::NoPen);
@@ -66,9 +67,6 @@ MsgCountComposedIcon::clone() const
 
 QList<QSize>
 MsgCountComposedIcon::availableSizes(QIcon::Mode mode, QIcon::State state)
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-  const
-#endif
 {
     Q_UNUSED(mode);
     Q_UNUSED(state);
@@ -97,11 +95,12 @@ MsgCountComposedIcon::pixmap(const QSize &size, QIcon::Mode mode, QIcon::State s
 
 TrayIcon::TrayIcon(const QString &filename, QWindow *parent)
   : QSystemTrayIcon(parent)
+  , icon(filename)
 {
 #if defined(Q_OS_MACOS) || defined(Q_OS_WIN)
-    setIcon(QIcon(filename));
+    setIcon(icon);
 #else
-    icon_ = new MsgCountComposedIcon(filename);
+    auto icon_ = new MsgCountComposedIcon(icon);
     setIcon(QIcon(icon_));
 #endif
 
@@ -116,12 +115,41 @@ TrayIcon::TrayIcon(const QString &filename, QWindow *parent)
 
     menu->addAction(viewAction_);
     menu->addAction(quitAction_);
+
+    QString toolTip = QLatin1String("nheko");
+    QString profile = UserSettings::instance()->profile();
+    if (!profile.isEmpty())
+        toolTip.append(QStringLiteral(" | %1").arg(profile));
+
+    setToolTip(toolTip);
 }
 
 void
 TrayIcon::setUnreadCount(int count)
 {
     qGuiApp->setBadgeNumber(count);
+    if (count != previousCount) {
+        QString toolTip = QLatin1String("nheko");
+        QString profile = UserSettings::instance()->profile();
+        if (!profile.isEmpty())
+            toolTip.append(QStringLiteral(" | %1").arg(profile));
+
+        if (count != 0)
+            toolTip.append(tr("\n%n unread message(s)", "", count));
+
+        setToolTip(toolTip);
+    }
+
+#if !defined(Q_OS_MACOS) && !defined(Q_OS_WIN)
+    if (count != previousCount) {
+        auto i      = new MsgCountComposedIcon(icon);
+        i->msgCount = count;
+        setIcon(QIcon(i));
+        previousCount = count;
+    }
+#else
+    (void)previousCount;
+#endif
 }
 
 #include "moc_TrayIcon.cpp"
