@@ -13,6 +13,7 @@
 #include <QBuffer>
 #include <QComboBox>
 #include <QCryptographicHash>
+#include <QFile>
 #include <QGuiApplication>
 #include <QImageReader>
 #include <QProcessEnvironment>
@@ -26,6 +27,7 @@
 #include <QWindow>
 #include <QXmlStreamReader>
 
+#include <fmt/ranges.h>
 #include <nlohmann/json.hpp>
 
 #include <cmark.h>
@@ -41,6 +43,9 @@
 #include "MatrixClient.h"
 #include "UserSettingsPage.h"
 
+static QString
+getQuoteBody(const RelatedInfo &related);
+
 //! Match widgets/events with a description message.
 namespace {
 template<class T>
@@ -266,7 +271,7 @@ utils::stripReplyFallbacks(const mtx::events::collections::TimelineEvents &event
     related.quoted_body = QString::fromStdString(mtx::accessors::body(event));
     related.quoted_body =
       QString::fromStdString(stripReplyFromBody(related.quoted_body.toStdString()));
-    related.quoted_body = utils::getQuoteBody(related);
+    related.quoted_body = getQuoteBody(related);
 
     // get quoted body and strip reply fallback
     related.quoted_formatted_body = mtx::accessors::formattedBodyWithFallback(event);
@@ -297,52 +302,57 @@ utils::codepointIsEmoji(uint code)
 QString
 utils::replaceEmoji(const QString &body)
 {
-    QString fmtBody;
-    fmtBody.reserve(body.size());
+    if constexpr (QT_VERSION >= QT_VERSION_CHECK(6, 9, 0)) {
+        return body;
+    } else {
+        QString fmtBody;
+        fmtBody.reserve(body.size());
 
-    QVector<uint> utf32_string = body.toUcs4();
+        QVector<uint> utf32_string = body.toUcs4();
 
-    bool insideFontBlock = false;
-    bool insideTag       = false;
-    for (auto &code : utf32_string) {
-        if (code == U'<')
-            insideTag = true;
-        else if (code == U'>')
-            insideTag = false;
+        bool insideFontBlock = false;
+        bool insideTag       = false;
+        for (auto &code : utf32_string) {
+            if (code == U'<')
+                insideTag = true;
+            else if (code == U'>')
+                insideTag = false;
 
-        if (!insideTag && utils::codepointIsEmoji(code)) {
-            if (!insideFontBlock) {
-                fmtBody += QStringLiteral("<font face=\"") % UserSettings::instance()->emojiFont() %
-                           (UserSettings::instance()->enlargeEmojiOnlyMessages()
-                              ? QStringLiteral("\" size=\"4\">")
-                              : QStringLiteral("\">"));
-                insideFontBlock = true;
-            } else if (code == 0xfe0f) {
-                // BUG(Nico):
-                // Workaround https://bugreports.qt.io/browse/QTBUG-97401
-                // See also https://github.com/matrix-org/matrix-react-sdk/pull/1458/files
-                // Nheko bug: https://github.com/Nheko-Reborn/nheko/issues/439
-                continue;
+            if (!insideTag && utils::codepointIsEmoji(code)) {
+                if (!insideFontBlock) {
+                    fmtBody += QStringLiteral("<font face=\"") %
+                               UserSettings::instance()->emojiFont() %
+                               (UserSettings::instance()->enlargeEmojiOnlyMessages()
+                                  ? QStringLiteral("\" size=\"4\">")
+                                  : QStringLiteral("\">"));
+                    insideFontBlock = true;
+                } else if (code == 0xfe0f) {
+                    // BUG(Nico):
+                    // Workaround https://bugreports.qt.io/browse/QTBUG-97401
+                    // See also https://github.com/matrix-org/matrix-react-sdk/pull/1458/files
+                    // Nheko bug: https://github.com/Nheko-Reborn/nheko/issues/439
+                    continue;
+                }
+            } else {
+                if (insideFontBlock) {
+                    fmtBody += QStringLiteral("</font>");
+                    insideFontBlock = false;
+                }
             }
-        } else {
-            if (insideFontBlock) {
-                fmtBody += QStringLiteral("</font>");
-                insideFontBlock = false;
+            if (QChar::requiresSurrogates(code)) {
+                QChar emoji[] = {static_cast<ushort>(QChar::highSurrogate(code)),
+                                 static_cast<ushort>(QChar::lowSurrogate(code))};
+                fmtBody.append(emoji, 2);
+            } else {
+                fmtBody.append(QChar(static_cast<ushort>(code)));
             }
         }
-        if (QChar::requiresSurrogates(code)) {
-            QChar emoji[] = {static_cast<ushort>(QChar::highSurrogate(code)),
-                             static_cast<ushort>(QChar::lowSurrogate(code))};
-            fmtBody.append(emoji, 2);
-        } else {
-            fmtBody.append(QChar(static_cast<ushort>(code)));
+        if (insideFontBlock) {
+            fmtBody += QStringLiteral("</font>");
         }
-    }
-    if (insideFontBlock) {
-        fmtBody += QStringLiteral("</font>");
-    }
 
-    return fmtBody;
+        return fmtBody;
+    }
 }
 
 void
@@ -1148,44 +1158,7 @@ utils::markdownToHtml(const QString &text, bool rainbowify_, bool noExtensions)
 }
 
 QString
-utils::getFormattedQuoteBody(const RelatedInfo &related, const QString &html)
-{
-    auto getFormattedBody = [related]() -> QString {
-        using MsgType = mtx::events::MessageType;
-
-        switch (related.type) {
-        case MsgType::File: {
-            return QStringLiteral("sent a file.");
-        }
-        case MsgType::Image: {
-            return QStringLiteral("sent an image.");
-        }
-        case MsgType::Audio: {
-            return QStringLiteral("sent an audio file.");
-        }
-        case MsgType::Video: {
-            return QStringLiteral("sent a video");
-        }
-        default: {
-            return escapeBlacklistedHtml(related.quoted_formatted_body);
-        }
-        }
-    };
-
-    return QStringLiteral("<mx-reply><blockquote><a "
-                          "href=\"https://matrix.to/#/%1/%2\">In reply "
-                          "to</a> <a href=\"https://matrix.to/#/%3\">%4</a><br"
-                          "/>%5</blockquote></mx-reply>")
-             .arg(related.room,
-                  QString::fromStdString(related.related_event),
-                  QUrl::toPercentEncoding(related.quoted_user),
-                  related.quoted_user.toHtmlEscaped(),
-                  getFormattedBody()) +
-           html;
-}
-
-QString
-utils::getQuoteBody(const RelatedInfo &related)
+getQuoteBody(const RelatedInfo &related)
 {
     using MsgType = mtx::events::MessageType;
 
@@ -2142,3 +2115,134 @@ utils::graduallyGlitchText(const QString &text)
 
     return result;
 }
+
+static QString
+mxidFromSegments(QStringView sigil, QStringView mxid)
+{
+    if (mxid.isEmpty())
+        return QString();
+
+    auto mxid_ = QUrl::fromPercentEncoding(mxid.toUtf8());
+
+    if (sigil == u"u") {
+        return "@" + mxid_;
+    } else if (sigil == u"roomid") {
+        return "!" + mxid_;
+    } else if (sigil == u"r") {
+        return "#" + mxid_;
+        //} else if (sigil == "group") {
+        //        return "+" + mxid_;
+    } else {
+        return QString();
+    }
+}
+
+std::optional<utils::MatrixUriParseResult>
+utils::parseMatrixUri(QString uri)
+{
+    QUrl uri_{uri};
+
+    // Convert matrix.to URIs to proper format
+    if (uri_.scheme() == QLatin1String("https") && uri_.host() == QLatin1String("matrix.to")) {
+        QString p = uri_.fragment(QUrl::FullyEncoded);
+        if (p.startsWith(QLatin1String("/")))
+            p.remove(0, 1);
+
+        auto temp = p.split(QStringLiteral("?"));
+        QString query;
+        if (temp.size() >= 2)
+            query = QUrl::fromPercentEncoding(temp.takeAt(1).toUtf8());
+
+        temp            = temp.first().split(QStringLiteral("/"));
+        auto identifier = QUrl::fromPercentEncoding(temp.takeFirst().toUtf8());
+        QString eventId = QUrl::fromPercentEncoding(temp.join('/').toUtf8());
+        if (!identifier.isEmpty()) {
+            if (identifier.startsWith(QLatin1String("@"))) {
+                QByteArray newUri = "matrix:u/" + QUrl::toPercentEncoding(identifier.remove(0, 1));
+                if (!query.isEmpty())
+                    newUri.append("?" + query.toUtf8());
+                uri_ = QUrl::fromEncoded(newUri);
+            } else if (identifier.startsWith(QLatin1String("#"))) {
+                QByteArray newUri = "matrix:r/" + QUrl::toPercentEncoding(identifier.remove(0, 1));
+                if (!eventId.isEmpty())
+                    newUri.append("/e/" + QUrl::toPercentEncoding(eventId.remove(0, 1)));
+                if (!query.isEmpty())
+                    newUri.append("?" + query.toUtf8());
+                uri_ = QUrl::fromEncoded(newUri);
+            } else if (identifier.startsWith(QLatin1String("!"))) {
+                QByteArray newUri =
+                  "matrix:roomid/" + QUrl::toPercentEncoding(identifier.remove(0, 1));
+                if (!eventId.isEmpty())
+                    newUri.append("/e/" + QUrl::toPercentEncoding(eventId.remove(0, 1)));
+                if (!query.isEmpty())
+                    newUri.append("?" + query.toUtf8());
+                uri_ = QUrl::fromEncoded(newUri);
+            }
+        }
+    }
+
+    // non-matrix URIs are not handled by us, return false
+    if (uri_.scheme() != QLatin1String("matrix"))
+        return {};
+
+    auto tempPath = uri_.path(QUrl::ComponentFormattingOption::FullyEncoded);
+    if (tempPath.startsWith('/'))
+        tempPath.remove(0, 1);
+    auto segments = QStringView(tempPath).split('/');
+
+    if (segments.size() != 2 && segments.size() != 4)
+        return {};
+
+    auto sigil1 = segments[0];
+    auto mxid1  = mxidFromSegments(sigil1, segments[1]);
+    if (mxid1.isEmpty())
+        return {};
+
+    QString mxid2;
+    QString sigil2;
+    if (segments.size() == 4 && segments[2] == QStringView(u"e")) {
+        if (segments[3].isEmpty())
+            return {};
+        else
+            mxid2 = "$" + QUrl::fromPercentEncoding(segments[3].toUtf8());
+        sigil2 = "$";
+    }
+
+    std::vector<std::string> vias;
+    QString action;
+
+    auto items =
+      uri_.query(QUrl::ComponentFormattingOption::FullyEncoded).split('&', Qt::SkipEmptyParts);
+    for (QString item : std::as_const(items)) {
+        if (item.startsWith(QLatin1String("action="))) {
+            action = item.remove(QStringLiteral("action="));
+        } else if (item.startsWith(QLatin1String("via="))) {
+            vias.push_back(QUrl::fromPercentEncoding(item.remove(QStringLiteral("via=")).toUtf8())
+                             .toStdString());
+        }
+    }
+
+    return MatrixUriParseResult{
+      .sigil1 = sigil1.toString(),
+      .mxid1  = std::move(mxid1),
+      .sigil2 = std::move(sigil2),
+      .mxid2  = std::move(mxid2),
+      .action = std::move(action),
+      .vias   = std::move(vias),
+    };
+}
+
+void
+utils::markFileAsFromWeb(const QString &path [[maybe_unused]])
+{
+#ifdef Q_OS_WINDOWS
+    QFile file(path + ":Zone.Identifier");
+    if (!file.open(QIODevice::Truncate | QIODevice::WriteOnly)) {
+        nhlog::net()->error("Failed to open alternate stream for {}", path.toStdString());
+        return;
+    }
+
+    file.write("[ZoneTransfer]\nZoneId=3");
+    file.close();
+#endif
+}
