@@ -147,14 +147,6 @@ escapeMentionMarkdown(QString input);
 QString
 escapeBlacklistedHtml(const QString &data);
 
-//! Generate a Rich Reply quote message
-QString
-getFormattedQuoteBody(const RelatedInfo &related, const QString &html);
-
-//! Get the body for the quote, depending on the event type.
-QString
-getQuoteBody(const RelatedInfo &related);
-
 //! Retrieve the color of the links based on the current theme.
 QString
 linkColor();
@@ -214,4 +206,16 @@ glitchText(const QString &text);
 
 QString
 graduallyGlitchText(const QString &text);
+
+struct MatrixUriParseResult
+{
+    QString sigil1, mxid1, sigil2, mxid2, action;
+    std::vector<std::string> vias;
+};
+
+std::optional<MatrixUriParseResult>
+parseMatrixUri(QString uri);
+
+void
+markFileAsFromWeb(const QString &file);
 }
