@@ -1,6 +1,4 @@
-// SPDX-FileCopyrightText: 2010 David Sansome <me@davidsansome.com>
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -154,6 +152,16 @@ directChat(const QString &userId)
         interface.call(QDBus::NoBlock, QStringLiteral("directChat"), userId);
 }
 
+QString
+statusMessage()
+{
+    if (QDBusInterface interface{QStringLiteral(NHEKO_DBUS_SERVICE_NAME), QStringLiteral("/")};
+        interface.isValid())
+        return QDBusReply<QString>{interface.call(QStringLiteral("statusMessage"))}.value();
+    else
+        return {};
+}
+
 void
 setStatusMessage(const QString &message)
 {
@@ -171,7 +179,6 @@ setStatusMessage(const QString &message)
  * http://www.clementine-player.org) and licensed under the GNU General Public
  * License, version 3 or later.
  *
- * SPDX-FileCopyrightText: 2010 David Sansome <me@davidsansome.com>
  */
 QDBusArgument &
 operator<<(QDBusArgument &arg, const QImage &image)
@@ -183,20 +190,17 @@ operator<<(QDBusArgument &arg, const QImage &image)
         return arg;
     }
 
-    QImage i = image.height() > 100 || image.width() > 100
-                 ? image.scaledToHeight(100, Qt::SmoothTransformation)
-                 : image;
-    i        = std::move(i).convertToFormat(QImage::Format_RGBA8888);
+    QImage i      = image.height() > 100 || image.width() > 100
+                      ? image.scaledToHeight(100, Qt::SmoothTransformation)
+                      : image;
+    bool hasAlpha = i.hasAlphaChannel();
+    i = std::move(i).convertToFormat(hasAlpha ? QImage::Format_RGBA8888 : QImage::Format_RGB888);
 
+    int channels = hasAlpha ? 4 : 3;
+    QByteArray arr(reinterpret_cast<const char *>(i.bits()), static_cast<int>(i.sizeInBytes()));
     arg.beginStructure();
-    arg << i.width();
-    arg << i.height();
-    arg << i.bytesPerLine();
-    arg << i.hasAlphaChannel();
-    int channels = i.hasAlphaChannel() ? 4 : 3;
-    arg << i.depth() / channels;
-    arg << channels;
-    arg << QByteArray(reinterpret_cast<const char *>(i.bits()), static_cast<int>(i.sizeInBytes()));
+    arg << i.width() << i.height() << (int)i.bytesPerLine() << i.hasAlphaChannel()
+        << i.depth() / channels << channels << arr;
     arg.endStructure();
 
     return arg;
@@ -224,3 +228,5 @@ operator>>(const QDBusArgument &arg, QImage &image)
 
     return arg;
 }
+
+#include "moc_NhekoDBusApi.cpp"
