@@ -1,5 +1,4 @@
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -86,7 +85,10 @@ Dock::Dock(QObject *parent)
 {
 }
 void
-Dock::setUnreadCount(const int)
+Dock::setUnreadCount(const int count)
 {
+    qGuiApp->setBadgeNumber(count);
 }
 #endif
+
+#include "moc_Dock.cpp"
