@@ -1,22 +1,18 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
 #include <QObject>
+#include <QQmlEngine>
 
-#include <mtx/responses/crypto.hpp>
+#include <mtxclient/crypto/client.hpp>
 
 #include "CacheCryptoStructs.h"
-#include "Logging.h"
-#include "MatrixClient.h"
-#include "Olm.h"
-#include "timeline/TimelineModel.h"
 
 class QTimer;
+class TimelineModel;
 
 using sas_ptr = std::unique_ptr<mtx::crypto::SAS>;
 
@@ -65,6 +61,9 @@ using sas_ptr = std::unique_ptr<mtx::crypto::SAS>;
 class DeviceVerificationFlow final : public QObject
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_UNCREATABLE("")
+
     Q_PROPERTY(QString state READ state NOTIFY stateChanged)
     Q_PROPERTY(Error error READ error NOTIFY errorChanged)
     Q_PROPERTY(QString userId READ getUserId CONSTANT)
@@ -103,6 +102,7 @@ public:
         KeyMismatch,
         Timeout,
         User,
+        AcceptedOnOtherDevice,
         OutOfOrder,
     };
     Q_ENUM(Error)
@@ -150,8 +150,6 @@ public:
     bool isSelfVerification() const;
     bool isMultiDeviceVerification() const { return deviceIds.size() > 1; }
 
-    void callback_fn(const UserKeyCache &res, mtx::http::RequestErr err, std::string user_id);
-
 public slots:
     //! unverifies a device
     void unverify();
@@ -230,32 +228,5 @@ private:
     bool keySent = false, macSent = false, acceptSent = false, startSent = false;
 
     template<typename T>
-    void send(T msg)
-    {
-        if (this->type == DeviceVerificationFlow::Type::ToDevice) {
-            mtx::requests::ToDeviceMessages<T> body;
-            msg.transaction_id = this->transaction_id;
-            for (const auto &d : deviceIds)
-                body[this->toClient][d.toStdString()] = msg;
-
-            http::client()->send_to_device<T>(
-              http::client()->generate_txn_id(), body, [](mtx::http::RequestErr err) {
-                  if (err)
-                      nhlog::net()->warn("failed to send verification to_device message: {} {}",
-                                         err->matrix_error.error,
-                                         static_cast<int>(err->status_code));
-              });
-        } else if (this->type == DeviceVerificationFlow::Type::RoomMsg && model_) {
-            if constexpr (!std::is_same_v<T, mtx::events::msg::KeyVerificationRequest>) {
-                msg.relations.relations.push_back(this->relation);
-                // Set synthesized to surpress the nheko relation extensions
-                msg.relations.synthesized = true;
-            }
-            (model_)->sendMessageEvent(msg, mtx::events::to_device_content_to_type<T>);
-        }
-
-        nhlog::net()->debug("Sent verification step: {} in state: {}",
-                            mtx::events::to_string(mtx::events::to_device_content_to_type<T>),
-                            state().toStdString());
-    }
+    void send(T msg);
 };
