@@ -1,6 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -8,6 +6,10 @@
 
 #include <QApplication>
 
+#include <nlohmann/json.hpp>
+
+#include <mtx/responses/common.hpp>
+
 #include "Cache.h"
 #include "Cache_p.h"
 #include "ChatPage.h"
@@ -18,19 +20,23 @@
 #include "timeline/TimelineViewManager.h"
 #include "ui/UIA.h"
 
-#include <mtx/responses/common.hpp>
-
 SelfVerificationStatus::SelfVerificationStatus(QObject *o)
   : QObject(o)
 {
     connect(ChatPage::instance(), &ChatPage::contentLoaded, this, [this] {
+        // We connect INSIDE a lambda, not A lambda...
         connect(cache::client(),
                 &Cache::selfVerificationStatusChanged,
                 this,
                 &SelfVerificationStatus::invalidate,
-                Qt::UniqueConnection);
+                Qt::UniqueConnection); // clazy:exclude=lambda-unique-connection
         cache::client()->markUserKeysOutOfDate({http::client()->user_id().to_string()});
     });
+
+    connect(ChatPage::instance(),
+            &ChatPage::initializeEmptyViews,
+            this,
+            &SelfVerificationStatus::invalidate);
 }
 
 void
@@ -105,7 +111,7 @@ SelfVerificationStatus::setupCrosssigning(bool useSSSS,
         http::client()->set_secret_storage_default_key(ssss->keyDescription.name,
                                                        [](mtx::http::RequestErr) {});
 
-        auto uploadSecret = [ssss](const std::string &key_name, const std::string &secret) {
+        auto uploadSecret = [ssss](std::string_view key_name, const std::string &secret) {
             mtx::secret_storage::Secret s;
             s.encrypted[ssss->keyDescription.name] =
               mtx::crypto::encrypt(secret, ssss->privateKey, key_name);
@@ -243,6 +249,8 @@ SelfVerificationStatus::verifyMasterKey()
     if (!devices.empty())
         ChatPage::instance()->timelineManager()->verificationManager()->verifyOneOfDevices(
           QString::fromStdString(this_user), std::move(devices));
+    else
+        nhlog::db()->info("No devices to verify");
 }
 
 void
@@ -281,7 +289,7 @@ SelfVerificationStatus::invalidate()
 {
     using namespace mtx::secret_storage;
 
-    nhlog::db()->info("Invalidating self verification status");
+    nhlog::db()->debug("Invalidating self verification status");
     if (!cache::isInitialized()) {
         nhlog::db()->warn("SelfVerificationStatus: cache not initialized");
         return;
@@ -347,3 +355,5 @@ SelfVerificationStatus::invalidate()
         return;
     }
 }
+
+#include "moc_SelfVerificationStatus.cpp"
