@@ -1,6 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -11,12 +9,16 @@
 #include "Cache.h"
 #include "ChatPage.h"
 #include "DeviceVerificationFlow.h"
+#include "Logging.h"
+#include "timeline/RoomlistModel.h"
+#include "timeline/TimelineModel.h"
 #include "timeline/TimelineViewManager.h"
 
 VerificationManager::VerificationManager(TimelineViewManager *o)
   : QObject(o)
   , rooms_(o->rooms())
 {
+    instance_ = this;
 }
 
 static bool
@@ -170,3 +172,5 @@ VerificationManager::verifyOneOfDevices(QString userid, std::vector<QString> dev
     this->dvList[flow->transactionId()] = flow;
     emit newDeviceVerificationRequest(flow.data());
 }
+
+#include "moc_VerificationManager.cpp"
