@@ -8,7 +8,6 @@
 #include <QCommandLineParser>
 #include <QDesktopServices>
 #include <QDir>
-#include <QFile>
 #include <QFontDatabase>
 #include <QLabel>
 #include <QLibraryInfo>
@@ -21,7 +20,7 @@
 
 // in theory we can enable this everywhere, but the header is missing on some of our CI systems and
 // it is too much effort to install.
-#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
+#if __has_include(<QtGui/qpa/qplatformwindow_p.h>)
 #include <QtGui/qpa/qplatformwindow_p.h>
 #endif
 
@@ -37,7 +36,6 @@
 #include "config/nheko.h"
 
 #if defined(Q_OS_MACOS)
-#include "emoji/MacHelper.h"
 #include "notifications/Manager.h"
 #endif
 
@@ -169,7 +167,7 @@ main(int argc, char *argv[])
 
     // this needs to be after setting the application name. Or how would we find our settings
     // file then?
-#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
+#if !defined(Q_OS_MACOS)
     if (qgetenv("QT_SCALE_FACTOR").size() == 0) {
         float factor = utils::scaleFactor();
 
@@ -248,7 +246,10 @@ main(int argc, char *argv[])
     if (!singleapp.isPrimaryInstance()) {
         auto token = qgetenv("XDG_ACTIVATION_TOKEN");
 
-#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
+#if __has_include(<QtGui/qpa/qplatformwindow_p.h>) && \
+        ((QT_VERSION >= QT_VERSION_CHECK(6, 7, 0) &&  QT_CONFIG(wayland)) || \
+         (QT_VERSION < QT_VERSION_CHECK(6, 7, 0) && defined(Q_OS_UNIX) && !defined(Q_OS_MACOS) \
+         && !defined(Q_OS_HAIKU)))
         // getting a valid activation token on wayland is a bit of a pain, it works most reliably
         // when you have an actual window, that has the focus...
         auto waylandApp = app.nativeInterface<QNativeInterface::QWaylandApplication>();
@@ -371,6 +372,13 @@ main(int argc, char *argv[])
     font.setPointSizeF(settings.lock()->fontSize());
 
     app.setFont(font);
+
+#if QT_VERSION >= QT_VERSION_CHECK(6, 9, 0)
+    if (auto emojiFont = settings.lock()->emojiFont(); !emojiFont.isEmpty()) {
+        QFontDatabase::addApplicationEmojiFontFamily(emojiFont);
+    }
+#endif
+
     settings.lock()->applyTheme();
 
     if (QLocale().language() == QLocale::C)
@@ -463,10 +471,6 @@ main(int argc, char *argv[])
       QStringLiteral("matrix"), ChatPage::instance(), "handleMatrixUri");
 
 #if defined(Q_OS_MACOS)
-    // Temporary solution for the emoji picker until
-    // nheko has a proper menu bar with more functionality.
-    MacHelper::initializeMenus();
-
     // Need to set up notification delegate so users can respond to messages from within the
     // notification itself.
     NotificationsManager::attachToMacNotifCenter();
