@@ -1,6 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -13,15 +11,13 @@
 #include <QObject>
 #include <QVariant>
 
+#include <mtx/common.hpp>
 #include <mtx/events/collections.hpp>
 #include <mtx/responses/messages.hpp>
 #include <mtx/responses/sync.hpp>
 
-#include "Reaction.h"
 #include "encryption/Olm.h"
 
-class TimelineModel;
-
 class EventStore final : public QObject
 {
     Q_OBJECT
@@ -29,10 +25,10 @@ class EventStore final : public QObject
 public:
     EventStore(std::string room_id, QObject *parent);
 
-    static void refetchOnlineKeyBackupKeys(TimelineModel *room);
+    void refetchOnlineKeyBackupKeys();
 
     // taken from QtPrivate::QHashCombine
-    static uint hashCombine(uint hash, uint seed)
+    static size_t hashCombine(uint hash, uint seed)
     {
         return seed ^ (hash + 0x9e3779b9 + (seed << 6) + (seed >> 2));
     };
@@ -41,9 +37,9 @@ public:
         std::string room;
         uint64_t idx;
 
-        friend uint qHash(const Index &i, uint seed = 0) noexcept
+        friend size_t qHash(const Index &i, size_t seed = 0) noexcept
         {
-            seed = hashCombine(qHashBits(i.room.data(), (int)i.room.size(), seed), seed);
+            seed = hashCombine(qHashBits(i.room.data(), i.room.size(), seed), seed);
             seed = hashCombine(qHash(i.idx, seed), seed);
             return seed;
         }
@@ -57,10 +53,10 @@ public:
     {
         std::string room, id;
 
-        friend uint qHash(const IdIndex &i, uint seed = 0) noexcept
+        friend size_t qHash(const IdIndex &i, size_t seed = 0) noexcept
         {
-            seed = hashCombine(qHashBits(i.room.data(), (int)i.room.size(), seed), seed);
-            seed = hashCombine(qHashBits(i.id.data(), (int)i.id.size(), seed), seed);
+            seed = hashCombine(qHashBits(i.room.data(), i.room.size(), seed), seed);
+            seed = hashCombine(qHashBits(i.id.data(), i.id.size(), seed), seed);
             return seed;
         }
 
@@ -75,12 +71,12 @@ public:
 
     // optionally returns the event or nullptr and fetches it, after which it emits a
     // relatedFetched event
-    mtx::events::collections::TimelineEvents *get(const std::string &id,
-                                                  std::string_view related_to,
-                                                  bool decrypt       = true,
-                                                  bool resolve_edits = true);
+    mtx::events::collections::TimelineEvents const *get(const std::string &id,
+                                                        std::string_view related_to,
+                                                        bool decrypt       = true,
+                                                        bool resolve_edits = true);
     // always returns a proper event as long as the idx is valid
-    mtx::events::collections::TimelineEvents *get(int idx, bool decrypt = true);
+    mtx::events::collections::TimelineEvents const *get(int idx, bool decrypt = true);
 
     QVariantList reactions(const std::string &event_id);
     std::vector<mtx::events::collections::TimelineEvents> edits(const std::string &event_id);
@@ -110,7 +106,7 @@ signals:
     void newEncryptedImage(mtx::crypto::EncryptedFile encryptionInfo);
     void eventFetched(std::string id,
                       std::string relatedTo,
-                      mtx::events::collections::TimelineEvents timeline);
+                      const mtx::events::collections::TimelineEvents &timeline);
     void oldMessagesRetrieved(const mtx::responses::Messages &);
     void fetchedMore();
 
@@ -122,13 +118,13 @@ signals:
     void updateFlowEventId(std::string event_id);
 
 public slots:
-    void addPending(mtx::events::collections::TimelineEvents event);
+    void addPending(const mtx::events::collections::TimelineEvents &event);
     void receivedSessionKey(const std::string &session_id);
     void clearTimeline();
     void enableKeyRequests(bool suppressKeyRequests_);
 
 private:
-    olm::DecryptionResult *
+    olm::DecryptionResult const *
     decryptEvent(const IdIndex &idx,
                  const mtx::events::EncryptedEvent<mtx::events::msg::Encrypted> &e);
 
