@@ -702,6 +702,8 @@ RoomlistModel::declineInvite(QString roomid)
 void
 RoomlistModel::leave(QString roomid, QString reason)
 {
+    // We want to leave in any case, even if this is an invite or similar.
+    ChatPage::instance()->leaveRoom(roomid, reason);
     if (models.contains(roomid)) {
         auto idx = roomidToIndex(roomid);
 
@@ -710,7 +712,6 @@ RoomlistModel::leave(QString roomid, QString reason)
             roomids.erase(roomids.begin() + idx);
             models.remove(roomid);
             endRemoveRows();
-            ChatPage::instance()->leaveRoom(roomid, reason);
         }
     }
 }
@@ -1014,7 +1015,7 @@ FilteredRoomlistModel::filterAcceptsRow(int sourceRow, const QModelIndex &) cons
                           ->data(sourceModel()->index(sourceRow, 0), RoomlistModel::Tags)
                           .toStringList();
 
-            for (const auto &t : tags)
+            for (const auto &t : std::as_const(tags))
                 if (hiddenTags.contains(t))
                     return false;
         }
@@ -1023,7 +1024,7 @@ FilteredRoomlistModel::filterAcceptsRow(int sourceRow, const QModelIndex &) cons
             auto parents = sourceModel()
                              ->data(sourceModel()->index(sourceRow, 0), RoomlistModel::ParentSpaces)
                              .toStringList();
-            for (const auto &t : parents)
+            for (const auto &t : std::as_const(parents))
                 if (hiddenSpaces.contains(t))
                     return false;
         }
@@ -1053,7 +1054,7 @@ FilteredRoomlistModel::filterAcceptsRow(int sourceRow, const QModelIndex &) cons
                           ->data(sourceModel()->index(sourceRow, 0), RoomlistModel::Tags)
                           .toStringList();
 
-            for (const auto &t : tags)
+            for (const auto &t : std::as_const(tags))
                 if (hiddenTags.contains(t))
                     return false;
         }
@@ -1062,7 +1063,7 @@ FilteredRoomlistModel::filterAcceptsRow(int sourceRow, const QModelIndex &) cons
             auto parents = sourceModel()
                              ->data(sourceModel()->index(sourceRow, 0), RoomlistModel::ParentSpaces)
                              .toStringList();
-            for (const auto &t : parents)
+            for (const auto &t : std::as_const(parents))
                 if (hiddenSpaces.contains(t))
                     return false;
         }
@@ -1091,7 +1092,7 @@ FilteredRoomlistModel::filterAcceptsRow(int sourceRow, const QModelIndex &) cons
             return false;
 
         if (!hiddenTags.empty()) {
-            for (const auto &t : tags)
+            for (const auto &t : std::as_const(tags))
                 if (t != filterStr && hiddenTags.contains(t))
                     return false;
         }
@@ -1100,7 +1101,7 @@ FilteredRoomlistModel::filterAcceptsRow(int sourceRow, const QModelIndex &) cons
             auto parents = sourceModel()
                              ->data(sourceModel()->index(sourceRow, 0), RoomlistModel::ParentSpaces)
                              .toStringList();
-            for (const auto &t : parents)
+            for (const auto &t : std::as_const(parents))
                 if (hiddenSpaces.contains(t))
                     return false;
         }
@@ -1130,13 +1131,13 @@ FilteredRoomlistModel::filterAcceptsRow(int sourceRow, const QModelIndex &) cons
                           ->data(sourceModel()->index(sourceRow, 0), RoomlistModel::Tags)
                           .toStringList();
 
-            for (const auto &t : tags)
+            for (const auto &t : std::as_const(tags))
                 if (hiddenTags.contains(t))
                     return false;
         }
 
         if (!hiddenSpaces.empty()) {
-            for (const auto &t : parents)
+            for (const auto &t : std::as_const(parents))
                 if (t != filterStr && hiddenSpaces.contains(t))
                     return false;
         }
