@@ -305,6 +305,7 @@ public:
     Q_INVOKABLE QString displayName(const QString &id) const;
     Q_INVOKABLE QString avatarUrl(const QString &id) const;
     Q_INVOKABLE QString formatDateSeparator(QDate date) const;
+    Q_INVOKABLE QString formatLaterSeparator(QDateTime prevDate, QDateTime date) const;
     Q_INVOKABLE QString formatTypingUsers(const QStringList &users, const QColor &bg);
     Q_INVOKABLE bool showAcceptKnockButton(const QString &id);
     Q_INVOKABLE void acceptKnock(const QString &id);
@@ -323,6 +324,7 @@ public:
       const mtx::events::StateEvent<mtx::events::msc2545::ImagePack> &event) const;
     Q_INVOKABLE QString formatPolicyRule(const QString &id) const;
     Q_INVOKABLE QVariantMap formatRedactedEvent(const QString &id);
+    Q_INVOKABLE bool showImage() const;
 
     Q_INVOKABLE void viewRawMessage(const QString &id);
     Q_INVOKABLE void forwardMessage(const QString &eventId, QString roomId);
@@ -359,7 +361,7 @@ public:
         auto list = events.reactions(event_id);
         std::vector<::Reaction> vec;
         vec.reserve(list.size());
-        for (const auto &r : list)
+        for (const auto &r : std::as_const(list))
             vec.push_back(r.value<Reaction>());
         return vec;
     }
