@@ -1,6 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -8,10 +6,13 @@
 
 #include <QColor>
 #include <QPalette>
+#include <QQmlEngine>
 
 class Theme final : public QPalette
 {
     Q_GADGET
+    QML_ANONYMOUS
+
     Q_PROPERTY(QColor sidebarBackground READ sidebarBackground CONSTANT)
     Q_PROPERTY(QColor alternateButton READ alternateButton CONSTANT)
     Q_PROPERTY(QColor separator READ separator CONSTANT)
