@@ -1,6 +1,4 @@
-// SPDX-FileCopyrightText: 2021 Nheko Contributors
-// SPDX-FileCopyrightText: 2022 Nheko Contributors
-// SPDX-FileCopyrightText: 2023 Nheko Contributors
+// SPDX-FileCopyrightText: Nheko Contributors
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -13,11 +11,12 @@
 #include "Cache_p.h"
 #include "ChatPage.h"
 #include "Logging.h"
+#include "MainWindow.h"
+#include "MatrixClient.h"
 #include "UserProfile.h"
 #include "Utils.h"
-#include "encryption/DeviceVerificationFlow.h"
 #include "encryption/VerificationManager.h"
-#include "mtx/responses/crypto.hpp"
+#include "timeline/RoomlistModel.h"
 #include "timeline/TimelineModel.h"
 #include "timeline/TimelineViewManager.h"
 #include "ui/UIA.h"
@@ -60,6 +59,25 @@ UserProfile::UserProfile(const QString &roomid,
           emit verificationStatiChanged();
       });
     fetchDeviceList(this->userid_);
+
+    if (userid != utils::localUser())
+        sharedRooms_ =
+          new RoomInfoModel(cache::client()->getCommonRooms(userid.toStdString()), this);
+    else
+        sharedRooms_ = new RoomInfoModel({}, this);
+
+    connect(ChatPage::instance(), &ChatPage::syncUI, this, [this](const mtx::responses::Sync &res) {
+        if (auto ignoreEv = std::ranges::find_if(
+              res.account_data.events,
+              [](const mtx::events::collections::RoomAccountDataEvents &e) {
+                  return std::holds_alternative<
+                    mtx::events::AccountDataEvent<mtx::events::account_data::IgnoredUsers>>(e);
+              });
+            ignoreEv != res.account_data.events.end()) {
+            // doesn't matter much if it was actually us
+            emit ignoredChanged();
+        }
+    });
 }
 
 QHash<int, QByteArray>
@@ -104,12 +122,53 @@ DeviceInfoModel::reset(const std::vector<DeviceInfo> &deviceList)
     endResetModel();
 }
 
+RoomInfoModel::RoomInfoModel(const std::map<std::string, RoomInfo> &raw, QObject *parent)
+  : QAbstractListModel(parent)
+{
+    for (const auto &e : raw)
+        roomInfos_.push_back(e);
+}
+
+QHash<int, QByteArray>
+RoomInfoModel::roleNames() const
+{
+    return {
+      {RoomId, "roomId"},
+      {RoomName, "roomName"},
+      {AvatarUrl, "avatarUrl"},
+    };
+}
+
+QVariant
+RoomInfoModel::data(const QModelIndex &index, int role) const
+{
+    if (!index.isValid() || index.row() >= (int)roomInfos_.size() || index.row() < 0)
+        return {};
+
+    switch (role) {
+    case RoomId:
+        return QString::fromStdString(roomInfos_[index.row()].first);
+    case RoomName:
+        return QString::fromStdString(roomInfos_[index.row()].second.name);
+    case AvatarUrl:
+        return QString::fromStdString(roomInfos_[index.row()].second.avatar_url);
+    default:
+        return {};
+    }
+}
+
 DeviceInfoModel *
 UserProfile::deviceList()
 {
     return &this->deviceList_;
 }
 
+RoomInfoModel *
+UserProfile::sharedRooms()
+{
+    return this->sharedRooms_;
+}
+
 QString
 UserProfile::userid()
 {
@@ -179,6 +238,60 @@ UserProfile::refreshDevices()
     fetchDeviceList(this->userid_);
 }
 
+bool
+UserProfile::ignored() const
+{
+    auto old = TimelineViewManager::instance()->getIgnoredUsers();
+    return old.contains(userid_);
+}
+
+void
+UserProfile::setIgnored(bool ignore)
+{
+    auto old = TimelineViewManager::instance()->getIgnoredUsers();
+    if (ignore) {
+        if (old.contains(userid_)) {
+            emit ignoredChanged();
+            return;
+        }
+        old.append(userid_);
+    } else {
+        if (!old.contains(userid_)) {
+            emit ignoredChanged();
+            return;
+        }
+        old.removeAll(userid_);
+    }
+
+    std::vector<mtx::events::account_data::IgnoredUser> content;
+    for (const QString &item : std::as_const(old)) {
+        content.push_back({item.toStdString()});
+    }
+
+    mtx::events::account_data::IgnoredUsers payload{.users{content}};
+
+    auto userid = userid_;
+
+    http::client()->put_account_data(payload, [userid](mtx::http::RequestErr e) {
+        if (e) {
+            MainWindow::instance()->showNotification(
+              tr("Failed to ignore \"%1\": %2")
+                .arg(userid, QString::fromStdString(e->matrix_error.error)));
+        }
+    });
+
+    if (ignore) {
+        const QHash<QString, RoomInfo> invites = cache::invites();
+        FilteredRoomlistModel *room_model      = FilteredRoomlistModel::instance();
+
+        for (auto room = invites.keyBegin(), end = invites.keyEnd(); room != end; room++) {
+            if (room_model->getRoomPreviewById(*room).inviterUserId() == userid) {
+                room_model->declineInvite(*room);
+            }
+        }
+    }
+}
+
 void
 UserProfile::fetchDeviceList(const QString &userID)
 {
@@ -300,10 +413,6 @@ UserProfile::banUser()
     ChatPage::instance()->banUser(roomid_, this->userid_, QLatin1String(""));
 }
 
-// void ignoreUser(){
-
-// }
-
 void
 UserProfile::kickUser()
 {
@@ -495,12 +604,18 @@ UserProfile::getGlobalProfileData()
                 emit avatarUrlChanged();
             });
 
+    connect(profProx.get(),
+            &UserProfileFetchProxy::failedToFetchProfile,
+            this,
+            &UserProfile::failedToFetchProfile);
+
     http::client()->get_profile(userid_.toStdString(),
                                 [prox = std::move(profProx), user = userid_.toStdString()](
                                   const mtx::responses::Profile &res, mtx::http::RequestErr err) {
                                     if (err) {
                                         nhlog::net()->warn("failed to retrieve profile info for {}",
                                                            user);
+                                        emit prox->failedToFetchProfile();
                                         return;
                                     }
 
@@ -513,3 +628,5 @@ UserProfile::openGlobalProfile()
 {
     emit manager->openGlobalUserProfile(userid_);
 }
+
+#include "moc_UserProfile.cpp"
